/* EXTENSION FILE */

/* dfadata - DFA named elements data routines */

#include "flexdef.h"

/**********************************************************
 * Methods for ActionListReferenceLists
 **********************************************************/

int actionListReferenceList_getPriority PROTO((int));

///////////////
// PUBLIC
///////////////

int newActionListReferenceList(int* actionLists, int size) {
	int i,n,hash = 0;
	
	for(i=0; i<size; i++) hash += actionLists[i];
	
	for(i=0; i<actionListReferenceLists_size; i++) {
		if(actionListReferenceList_hash_values[i] == hash) {
			ActionListReferenceList *oldActionListReferenceList = &actionListReferenceLists[i];
			int old_size = oldActionListReferenceList->size;
			if(old_size == size) {
				for(n=0; n<size; n++) {
					if(oldActionListReferenceList->elements[n] != actionLists[n]) break;
				}
				
				if(n == size) return i;
			}
		}
	}
	
	if(actionListReferenceLists_size == actionListReferenceLists_size_max) {
		++num_reallocs;
		actionListReferenceLists_size_max += MAX_ACTION_LIST_REFERENCE_LIST_SIZE_INCREMENT;
		actionListReferenceLists = reallocate_actionListReferenceList_array(actionListReferenceLists,actionListReferenceLists_size_max);
		actionListReferenceList_hash_values = reallocate_integer_array(actionListReferenceList_hash_values,actionListReferenceLists_size_max);
	}
	
	ActionListReferenceList *newActionListReferenceList = &actionListReferenceLists[actionListReferenceLists_size];
	newActionListReferenceList->size = size;
	newActionListReferenceList->elements = allocate_integer_array(size);
	
	for(i=0; i<size; i++) newActionListReferenceList->elements[i] = actionLists[i];
	
	actionListReferenceList_hash_values[actionListReferenceLists_size] = hash;
	
	return actionListReferenceLists_size++;
}

void actionListReferenceList_print(int list, FILE* file) {
	int i, size = actionListReferenceLists[list].size;
	
	fprintf(file,"[");
	
	for(i=0; i<size; i++) {
		if(i != 0) fprintf(file,",");
		
		int element = actionListReferenceLists[list].elements[i];
		
		fprintf(file,"(%i)-",element);
		actionList_print(element,file);
	}
	
	fprintf(file,"]");
}

///////////////
// PRIVATE
///////////////

int actionListReferenceList_getPriority(int list) {
	int i, priority = 0, size = actionListReferenceLists[list].size;
	for(i=0; i<size; i++) priority += action_lists[actionListReferenceLists[list].elements[i]].priority;
	return priority;
}



/**********************************************************
 * Methods for ActionListReferenceListSets
 **********************************************************/

void actionListReferenceList_optimize PROTO((int));
void actionListReferenceList_finalize PROTO((int));

///////////////
// PUBLIC
///////////////

int newActionListReferenceListSet(int firstElement, int firstSource, int firstTarget) {
	if(actionListReferenceListSets_size == actionListReferenceListSets_size_max) {
		++num_reallocs;
		actionListReferenceListSets_size_max += MAX_ACTION_LIST_REFERENCE_LIST_SET_SIZE_INCREMENT;
		actionListReferenceListSets = reallocate_actionListReferenceListSet_array(actionListReferenceListSets,actionListReferenceListSets_size_max);
	}
	
	ActionListReferenceListSet *newActionListReferenceListSet = &actionListReferenceListSets[actionListReferenceListSets_size];
	newActionListReferenceListSet->size = 1;
	newActionListReferenceListSet->max_size = INITIAL_ACTION_LIST_REFERENCE_LIST_SET_ELEMENT_SIZE;
	newActionListReferenceListSet->elements = allocate_integer_array(INITIAL_ACTION_LIST_REFERENCE_LIST_SET_ELEMENT_SIZE);
	newActionListReferenceListSet->targets = allocate_integer_array(INITIAL_ACTION_LIST_REFERENCE_LIST_SET_ELEMENT_SIZE);
	newActionListReferenceListSet->sources = allocate_integer_array(INITIAL_ACTION_LIST_REFERENCE_LIST_SET_ELEMENT_SIZE);
	newActionListReferenceListSet->elements[0] = firstElement;
	newActionListReferenceListSet->targets[0] = firstTarget;
	newActionListReferenceListSet->sources[0] = firstTarget;
	
	return actionListReferenceListSets_size++;
}

void actionListReferenceListSet_add(int set_index, int element, int source, int target) {
	ActionListReferenceListSet* set = &actionListReferenceListSets[set_index];
	int i, size = set->size;

	if(size == set->max_size) {
		++num_reallocs;
		set->max_size += MAX_ACTION_LIST_REFERENCE_LIST_SET_ELEMENT_SIZE_INCREMENT;
		set->elements = reallocate_integer_array(set->elements,set->max_size);
		set->targets = reallocate_integer_array(set->targets,set->max_size);
		set->sources = reallocate_integer_array(set->sources,set->max_size);
	}
	
	set->elements[size] = element;
	set->targets[size] = target;
	set->sources[size] = source;
	set->size = size + 1;
}

void actionListReferenceListSet_finish(int set_index) {
	actionListReferenceList_optimize(set_index);
	actionListReferenceList_finalize(set_index);
	// bubble(set->elements-1,newSize); // Sorting for more beautiful debug output
}

void actionListReferenceListSet_print(int set_index, FILE* file) {
	ActionListReferenceListSet* set;
	int i, size;
	
	if(set_index == UNUSED) {
		fprintf(file,"UNUSED");
		return;
	}
	
	set = &actionListReferenceListSets[set_index];
	size = set->size;
	
	fprintf(file,"{");
	
	for(i=0; i<size; i++) {
		if(i != 0) fprintf(file,",");
		
		int element = set->elements[i];
		actionListReferenceList_print(element,file);
	}
	
	fprintf(file,"}");
}

///////////////
// PRIVATE
///////////////

void actionListReferenceList_optimize(int set_index) {
	ActionListReferenceListSet* set = &actionListReferenceListSets[set_index];
	register int i,n,size = set->size, newSize = 0;
	if(size < 2) return;
	
	int *elements = set->elements;
	int *sources = set->sources;
	int *targets = set->targets;
	int *priorities = allocate_integer_array(size);
	
	for(i=0; i<size; i++) {
		priorities[i] = actionListReferenceList_getPriority(elements[i]);
	}
	
	int *newSetIndexes = allocate_integer_array(size);
	
	for(i=0; i<size; i++) {
		int element = elements[i];
		int source = sources[i];
		int target = targets[i];
		int priority = priorities[i];
		int loop_count = newSize;
		
		for(n=0; n<loop_count; n++) {
			if(newSetIndexes[n] == UNUSED) continue;
			int existingElement = newSetIndexes[n];
			
			if(targets[existingElement] == target && sources[existingElement] == source) {
				int existingPriority = priorities[existingElement];
				
				if(existingPriority == priority) newSetIndexes[newSize++] = i;
				else if(existingPriority < priority) {
					newSetIndexes[n] = i;
					
					for(n++; n<loop_count; n++) {
						existingElement = newSetIndexes[n];
						if(targets[existingElement] == target && sources[existingElement] == source) {
							newSetIndexes[n] = UNUSED;
						}
					}
				}
				break;
			}
		}
		
		if(n == newSize) newSetIndexes[newSize++] = i;
	}
	
	flex_free(priorities);
	
	size = newSize;
	newSize = 0;
	
	for(i=0; i<size; i++) {
		int element = newSetIndexes[i];
		if(element != UNUSED) elements[newSize++] = elements[element];
	}
	
	flex_free(newSetIndexes);
	
	set->size = newSize;
}

void actionListReferenceList_finalize(int set_index) {
	ActionListReferenceListSet* set = &actionListReferenceListSets[set_index];
	register int i,n, newSize = 0, size = set->size;
	if(size < 2) return;
	
	int *elements = set->elements;
	
	for(i=0; i<size; i++) {
		int element = elements[i];
		int exists = false;
		int element_size = actionListReferenceLists[element].size;
		int *element_elements = actionListReferenceLists[element].elements;
		
		for(n=0; n<newSize; n++) {
			int existingElement = elements[n];
			if(element != existingElement) {
				if(actionListReferenceList_hash_values[element] == actionListReferenceList_hash_values[existingElement]) {
					if(element_size == actionListReferenceLists[existingElement].size) {
						int i,*existingElement_elements = actionListReferenceLists[existingElement].elements;
						
						for(i=0; i<element_size; i++) {
							if(element_elements[i] != existingElement_elements[i]) break;
						}
						
						if(i == element_size) {
							exists = true;
							break;
						}
					}
				}
			}
			else {
				exists = true;
				break;
			}
		}
		
		if(!exists) elements[newSize++] = element;
	}
	
	set->size = newSize;
}



/**********************************************************
 * Methods for ActionListReferenceSets
 **********************************************************/

int newActionListReferenceSet(int listSet_index) {
	ActionListReferenceListSet* listSet = &actionListReferenceListSets[listSet_index];
	int i,n,index,size = listSet->size;
	
	for(i=0; i<actionListReferenceSets_size; i++) {
		int old_listSet_index = actionListReferenceSet_sources[i];
		if(old_listSet_index == listSet_index) return i;
		
		ActionListReferenceListSet *old_listSet = &actionListReferenceListSets[old_listSet_index];
		int old_size = old_listSet->size;
		if(old_size == size) {
			for(n=0; n<size; n++) {
				if(old_listSet->elements[n] != listSet->elements[n]) break;
			}
			
			if(n == size) return i;
		}
	}
	
	if(actionListReferenceSets_size == actionListReferenceSets_size_max) {
		++num_reallocs;
		actionListReferenceSets_size_max += MAX_ACTION_LIST_REFERENCE_SET_SIZE_INCREMENT;
		actionListReferenceSets = reallocate_actionListReferenceSet_array(actionListReferenceSets,actionListReferenceSets_size_max);
		actionListReferenceSet_sources = reallocate_integer_array(actionListReferenceSet_sources,actionListReferenceSets_size_max);
	}
	
	ActionListReferenceSet *newActionListReferenceSet = &actionListReferenceSets[actionListReferenceSets_size];
	newActionListReferenceSet->epsilon_set = false;
	newActionListReferenceSet->elements = allocate_integer_array(size);
	
	actionListReferenceSet_sources[actionListReferenceSets_size] = listSet_index;
	
	index = 0;
	for(i=0; i<size; i++) {
		int newOne = newRelevantActionListReference(listSet->elements[i]);
		if(newOne == -1) newActionListReferenceSet->epsilon_set = true;
		else newActionListReferenceSet->elements[index++] = newOne;
	}
	
	newActionListReferenceSet->size = index;
	
	return actionListReferenceSets_size++;
}

void actionListReferenceSet_print(int set_index, FILE* file) {
	ActionListReferenceSet* set;
	int i, size, epsilon_set;
	
	if(set_index == UNUSED) {
		fprintf(file,"UNUSED");
		return;
	}
	
	set = &actionListReferenceSets[set_index];
	size = set->size;
	epsilon_set = set->epsilon_set;
	
	fprintf(file,"%i-{",set_index);
	
	if(epsilon_set) fprintf(file,"e");
	
	for(i=0; i<size; i++) {
		if(i != 0 || epsilon_set) fprintf(file,",");
		
		int element = set->elements[i];
		fprintf(file,"(%i)-",element);
		actionList_print(relevant_actionListReferences[element],file);
	}
	
	fprintf(file,"}");
}

int newRelevantActionListReference(int list) {
	int i;
	
	for(i=0; i<relevant_size; i++) {
		if(relevant_actionListReference_sources[i] == list) return i;
	}
	
	int listSize = actionListReferenceLists[list].size;
	int actionList, first, last;
	
	if(listSize == 0) return -1;
	
	if(listSize == 1) {
		actionList = actionListReferenceLists[list].elements[0];
		first = actionList;
		last = actionList;
	}
	else {
		actionList = newActionListFromActionListReferenceList(list);
		first = actionListReferenceLists[list].elements[0];
		last = actionListReferenceLists[list].elements[actionListReferenceLists[list].size-1];
	}
	
	if(relevant_size == relevant_size_max) {
		relevant_size_max += MAX_RELEVANT_SIZE_INCREMENT;
		relevant_actionListReferences = reallocate_integer_array(relevant_actionListReferences,relevant_size_max);
		relevant_actionListReference_sources = reallocate_integer_array(relevant_actionListReference_sources,relevant_size_max);
		relevant_actionListReference_firsts = reallocate_integer_array(relevant_actionListReference_firsts,relevant_size_max);
		relevant_actionListReference_lasts = reallocate_integer_array(relevant_actionListReference_lasts,relevant_size_max);
	}
	
	relevant_actionListReferences[relevant_size] = actionList;
	relevant_actionListReference_sources[relevant_size] = list;
	relevant_actionListReference_firsts[relevant_size] = first;
	relevant_actionListReference_lasts[relevant_size] = last;
	
	return relevant_size++;
}



/**********************************************************
 * Methods for ntod()
 **********************************************************/

void add_actions PROTO((int,int,int*));

#define ADD_ACTION(action_set_ptr,target) { \
	int newList = newActionListReferenceList(action_buffer,action_buffer_size); \
	if(*action_set_ptr == UNUSED) *action_set_ptr = newActionListReferenceListSet(newList,state_buffer[0],target); \
	else actionListReferenceListSet_add(*action_set_ptr,newList,state_buffer[0],target); \
}

void evaluate_states_actions(int ds_state, int* state_actions) {
	int *nset_origin = dss_origin[ds_state];
	int i, nset_origin_size = nset_origin[0];
	
	action_buffer_size = 0;
	state_buffer_size = 0;
	
	for(i=1; i<=numecs; i++) state_actions[i] = UNUSED;
	
	for(i=1; i<=nset_origin_size; i++) add_actions(ds_state,nset_origin[i],state_actions);
	
	for(i=1; i<=numecs; i++) {
		int set = state_actions[i];
		if(set != UNUSED) actionListReferenceListSet_finish(set);
	}
	
	if(reject) {
		int accept_set_size = accsiz[ds_state];
		
		for(i=1; i<=accept_set_size; i++) {
			int set = dfaacc_action_list_sets[ds_state].dfaacc_set_actions[i];
			if(set != UNUSED) actionListReferenceListSet_finish(set);
		}
	}
	else {
		if(dfaacc[ds_state].dfaacc_state != 0) {
			int set = dfaacc_action_list_sets[ds_state].dfaacc_state_actions;
			if(set != UNUSED) actionListReferenceListSet_finish(set);
		}
	}
}

void add_actions(int ds_state, int state, int* state_actions) {
	int i,transsym,tsp,accept_number,actionList;
	
	for(i=0; i<state_buffer_size; i++) if(state_buffer[i] == state) return;

	transsym = transchar[state];

	if (transsym == SYM_EPSILON) {
		tsp = trans1[state];

		if (tsp != NO_TRANSITION) {
			actionList = nfa_action_list_references[state];
			state_buffer[state_buffer_size++] = state;
			
			if(actionList != UNUSED) action_buffer[action_buffer_size++] = actionList;
			
			add_actions(ds_state,tsp,state_actions);
			
			tsp = trans2[state];
			if (tsp != NO_TRANSITION) {
				add_actions(ds_state,tsp,state_actions);
			}

			if(actionList != UNUSED) action_buffer_size--;
			state_buffer_size--;
		}
	}
	else {
		actionList = nfa_action_list_references[state];
		
		state_buffer[state_buffer_size++] = state;
		if(actionList != UNUSED) action_buffer[action_buffer_size++] = actionList;
		
		if(transsym > 0) {
			int ec = ecgroup[transsym];
			ADD_ACTION(&state_actions[ec],trans1[state]);
		}
		else {
			int k;
			int lenccl = ccllen[-transsym];
			int cclp = cclmap[-transsym];
			
			if(cclng[-transsym]) {
				int j = 0;
				for(k = 0; k<lenccl; k++) {
					int ec = ccltbl[cclp + k];
					if (ec == 0) ec = NUL_ec;
					for (j++; j<ec; j++)
						ADD_ACTION(&state_actions[j],trans1[state]);
				}
				for (j++; j<=numecs; j++)
					ADD_ACTION(&state_actions[j],trans1[state]);
			}
			else {
				for (k = 0; k<lenccl; k++) {
					int ec = ccltbl[cclp + k];
					if (ec == 0) ec = NUL_ec;
					ADD_ACTION(&state_actions[ec],trans1[state]);
				}
			}
		}
		
		if(actionList != UNUSED) action_buffer_size--;
		state_buffer_size--;
	}
	
	accept_number = accptnum[state];
	
	if(accept_number != NIL) {
		actionList = nfa_action_list_references[state];
		if(actionList != UNUSED) action_buffer[action_buffer_size++] = actionList;
		
		if(reject) {
			int* accept_set = dfaacc[ds_state].dfaacc_set;
			int accept_set_size = dfasiz[ds_state];
			
			for(i=1; i<=accept_set_size; i++) {
				if(accept_set[i] == accept_number) {
					ADD_ACTION(&dfaacc_action_list_sets[ds_state].dfaacc_set_actions[i],0);
					break;
				}
			}
		}
		else if(accept_number == dfaacc[ds_state].dfaacc_state) {
			ADD_ACTION(&dfaacc_action_list_sets[ds_state].dfaacc_state_actions,0);
		}
		
		if(actionList != UNUSED) action_buffer_size--;
	}
}

void evaluate_start_follow_elements PROTO((int));
void evaluate_follow_elements PROTO((int,int,int*));
void add_int_to_int_set PROTO((int*,int**,int));
void add_int_set_to_int_set PROTO((int*,int**,int,int*));

void generateReferencesMapFromNFA() {
	int i, n, followed_set_size;
	int* followed_set;
	
	canFollowStart = allocate_integer_array(relevant_size);
	for(i=0; i<relevant_size; i++) canFollowStart[i] = false;
	
	canBeFollowedBy_sizes = allocate_integer_array(relevant_size);
	for(i=0; i<relevant_size; i++) canBeFollowedBy_sizes[i] = 0;
	
	canBeFollowedBy = allocate_int_ptr_array(relevant_size);
	for(i=0; i<relevant_size; i++) canBeFollowedBy[i] = NIL;
	
	followedByAccept_sizes = allocate_integer_array(num_rules + 1);
	for(i=1; i<=num_rules; i++) followedByAccept_sizes[i] = 0;
	
	followedByAccept = allocate_int_ptr_array(num_rules + 1);
	for(i=1; i<=num_rules; i++) followedByAccept[i] = NIL;
	
	startFollowedByAccept = allocate_integer_array(num_rules + 1);
	for(i=1; i<=num_rules; i++) startFollowedByAccept[i] = false;
	
	followed_set = allocate_integer_array(relevant_size);
	
	for(i=1; i<=numberOfStartStates; i++) {
		int nfaStartState = dss_origin[i][1];
		state_buffer_size = 0;
		evaluate_start_follow_elements(nfaStartState);
	}
	
	for(i=1; i<=lastnfa; i++) {
		int actionList = nfa_action_list_references[i];
		
		if(actionList != UNUSED) {
			int next = trans1[i];
			if(next != NO_TRANSITION) {
				
				followed_set_size = 0;
				for(n=0; n<relevant_size; n++) {
					if(relevant_actionListReference_lasts[n] == actionList) {
						followed_set[followed_set_size++] = n;
					}
				}
				
				if(followed_set_size > 0) {
					state_buffer_size = 0;
					evaluate_follow_elements(next,followed_set_size,followed_set);
					
					next = trans2[i];
					if(next != NO_TRANSITION) {
						state_buffer_size = 0;
						evaluate_follow_elements(next,followed_set_size,followed_set);
					}
				}
			}
		}
	}
	
	flex_free(followed_set);
}

void evaluate_start_follow_elements(int state) {
	int i, actionList = nfa_action_list_references[state];
	
	if(actionList != UNUSED) {
		for(i=0; i<relevant_size; i++) {
			if(relevant_actionListReference_firsts[i] == actionList) {
				canFollowStart[i] = true;
			}
		}
	}
	else {
		int next = trans1[state];
		if(next != NO_TRANSITION) {
			for(i=0; i<state_buffer_size; i++) if(state_buffer[i] == state) return;
			
			state_buffer[state_buffer_size++] = state;
			
			evaluate_start_follow_elements(next);
			
			next = trans2[state];
			if(next != NO_TRANSITION) {
				evaluate_start_follow_elements(next);
			}
			
			state_buffer_size--;
		}
	}
	
	int accept_number = accptnum[state];
	if(accept_number != NIL) {
		startFollowedByAccept[accept_number] = true;
	}
}

void evaluate_follow_elements(int state, int followed_set_size, int *followed_set) {
	int i, actionList = nfa_action_list_references[state];
	
	if(actionList != UNUSED) {
		for(i=0; i<relevant_size; i++) {
			if(relevant_actionListReference_firsts[i] == actionList) {
				add_int_set_to_int_set(&canBeFollowedBy_sizes[i],&canBeFollowedBy[i],followed_set_size,followed_set);
			}
		}
	}
	else {
		int next = trans1[state];
		if(next != NO_TRANSITION) {
			for(i=0; i<state_buffer_size; i++) if(state_buffer[i] == state) return;
			
			state_buffer[state_buffer_size++] = state;
			
			evaluate_follow_elements(next,followed_set_size,followed_set);
			
			next = trans2[state];
			if(next != NO_TRANSITION) {
				evaluate_follow_elements(next,followed_set_size,followed_set);
			}
			
			state_buffer_size--;
		}
	}
	
	int accept_number = accptnum[state];
	if(!(accept_number & YY_TRAILING_HEAD_MASK) && accept_number != NIL) {
		if(actionList != UNUSED) {
			for(i=0; i<relevant_size; i++) {
				if(relevant_actionListReference_firsts[i] == actionList) {
					add_int_set_to_int_set(&canBeFollowedBy_sizes[i],&canBeFollowedBy[i],followed_set_size,followed_set);
				}
				if(relevant_actionListReference_lasts[i] == actionList) {
					add_int_to_int_set(&followedByAccept_sizes[accept_number],&followedByAccept[accept_number],i);
				}
			}
		}
		else {
			add_int_set_to_int_set(&followedByAccept_sizes[accept_number],&followedByAccept[accept_number],followed_set_size,followed_set);
		}
	}
}

void add_int_to_int_set(int *size_ptr, int **set_ptr, int followed) {
	int i,size = *size_ptr, *set = *set_ptr;
	
	for(i=0; i<size; i++) {
		if(set[i] == followed) return;
	}

	if(size%FOLLOWED_BY_SET_INITIAL_SIZE_AND_INCREMENT == 0) {
		if(size == 0) {
			set = *set_ptr = allocate_integer_array(FOLLOWED_BY_SET_INITIAL_SIZE_AND_INCREMENT);
		}
		else {
			++num_reallocs;
			set = *set_ptr = reallocate_integer_array(*set_ptr,size+FOLLOWED_BY_SET_INITIAL_SIZE_AND_INCREMENT);
		}
	}
	
	set[size] = followed;
	*size_ptr = *size_ptr + 1;
}

void add_int_set_to_int_set(int *target_size_ptr, int **target_set_ptr, int source_size, int *source_set) {
	int i;
	
	for(i=0; i<source_size; i++) {
		add_int_to_int_set(target_size_ptr,target_set_ptr,source_set[i]);
	}
}

void printReferencesMap(FILE* file) {
	int i,n;
	
	if(!ACTIONS_USED) return;
	
	fprintf(file,"\n\nReferences Map Dump:\n\nAccepts following start: [");
	
	n = false;
	for(i=1; i<=num_rules; i++) {
		if(startFollowedByAccept[i]) {
			if(n) fprintf(file,",");
			fprintf(file,"%i",i);
			n = true;
		}
	}
	
	fprintf(file,"]\nAction lists following start: [");
	
	n = false;
	for(i=0; i<relevant_size; i++) {
		if(canFollowStart[i]) {
			if(n) fprintf(file,",");
			fprintf(file,"%i",i);
			n = true;
		}
	}
	
	fprintf(file,"]\n\nAction lists following action lists:\n");
	
	for(i=0; i<relevant_size; i++) {
		int size = canBeFollowedBy_sizes[i];
		if(size != 0) {
			fprintf(file,"%i <- [",i);
			for(n=0; n<size; n++) {
				if(n != 0) fprintf(file,",");
				fprintf(file,"%i",canBeFollowedBy[i][n]);
			}
			fprintf(file,"]\n");
		}
	}
	
	fprintf(file,"\nAccepts following action lists:\n");
	
	for(i=1; i<=num_rules; i++) {
		int size = followedByAccept_sizes[i];
		if(size != 0) {
			fprintf(file,"[%i] <- [",i);
			for(n=0; n<size; n++) {
				if(n != 0) fprintf(file,",");
				fprintf(file,"%i",followedByAccept[i][n]);
			}
			fprintf(file,"]\n");
		}
	}
	
	fprintf(file,"\n\n");
}


/**********************************************************
 * ActionList compressing
 **********************************************************/

void compress_states_action(int ds_state,int* state_action_list_sets, int* state_action_sets) {
	int i;
	
	for(i=0; i<=numecs; i++) {
		if(state_action_list_sets[i] == UNUSED) state_action_sets[i] = UNUSED;
		else state_action_sets[i] = newActionListReferenceSet(state_action_list_sets[i]);
	}
	
	if(reject) {
		int accept_set_size = accsiz[ds_state];
		
		for(i=1; i<=accept_set_size; i++) {
			int listSet = dfaacc_action_list_sets[ds_state].dfaacc_set_actions[i];
			if(listSet == UNUSED) dfaacc_action_sets[ds_state].dfaacc_set_actions[i] = UNUSED;
			else dfaacc_action_sets[ds_state].dfaacc_set_actions[i] = newActionListReferenceSet(listSet);
		}
	}
	else {
		if(dfaacc[ds_state].dfaacc_state != 0) {
			int listSet = dfaacc_action_list_sets[ds_state].dfaacc_state_actions;
			if(listSet == UNUSED) dfaacc_action_sets[ds_state].dfaacc_state_actions = UNUSED;
			else dfaacc_action_sets[ds_state].dfaacc_state_actions = newActionListReferenceSet(listSet);
		}
	}
}
